@@ -50,36 +50,6 @@ class CryptoConfigTest: public QGpgMETest
     Q_OBJECT
 
 private Q_SLOTS:
-    void testKeyserver()
-    {
-        // Repeatedly set a config value and clear it
-        // this was broken at some point so it gets a
-        // unit test.
-        for (int i = 0; i < 10; i++) {
-            auto conf = cryptoConfig();
-            QVERIFY(conf);
-            auto entry = conf->entry(QStringLiteral("gpg"),
-                    QStringLiteral("Keyserver"),
-                    QStringLiteral("keyserver"));
-            QVERIFY(entry);
-            const QString url(QStringLiteral("hkp://foo.bar.baz"));
-            entry->setStringValue(url);
-            conf->sync(false);
-            conf->clear();
-            entry = conf->entry(QStringLiteral("gpg"),
-                    QStringLiteral("Keyserver"),
-                    QStringLiteral("keyserver"));
-            QCOMPARE (entry->stringValue(), url);
-            entry->setStringValue(QString());
-            conf->sync(false);
-            conf->clear();
-            entry = conf->entry(QStringLiteral("gpg"),
-                    QStringLiteral("Keyserver"),
-                    QStringLiteral("keyserver"));
-            QCOMPARE (entry->stringValue(), QString());
-        }
-    }
-
     void testDefault()
     {
         if (GpgME::engineInfo(GpgME::GpgEngine).engineVersion() < "2.2.0") {
@@ -87,27 +57,24 @@ private Q_SLOTS:
             // be unsupported in older versions.
             return;
         }
-        // First set compliance to de-vs
         auto conf = cryptoConfig();
         QVERIFY(conf);
-        auto entry = conf->entry(QStringLiteral("gpg"),
-                QStringLiteral("Configuration"),
-                QStringLiteral("compliance"));
+        auto entry = conf->entry(QStringLiteral("gpg"), QStringLiteral("compliance"));
         QVERIFY(entry);
+        const auto defaultValue = entry->defaultValue().toString();
+        QCOMPARE(defaultValue, QStringLiteral("gnupg"));
+
         entry->setStringValue("de-vs");
         conf->sync(true);
         conf->clear();
-        entry = conf->entry(QStringLiteral("gpg"),
-                QStringLiteral("Configuration"),
-                QStringLiteral("compliance"));
+        entry = conf->entry(QStringLiteral("gpg"), QStringLiteral("compliance"));
         QCOMPARE(entry->stringValue(), QStringLiteral("de-vs"));
+
         entry->resetToDefault();
         conf->sync(true);
         conf->clear();
-        entry = conf->entry(QStringLiteral("gpg"),
-                QStringLiteral("Configuration"),
-                QStringLiteral("compliance"));
-        QCOMPARE(entry->stringValue(), QStringLiteral("gnupg"));
+        entry = conf->entry(QStringLiteral("gpg"), QStringLiteral("compliance"));
+        QCOMPARE(entry->stringValue(), defaultValue);
     }
 
     void initTestCase()
