@@ -35,7 +35,12 @@
 /* Local operation data.  */
 typedef struct
 {
-  gpg_error_t err;  /* Error encountered during the export.  */
+  /* The error code from a FAILURE status line or 0.  */
+  gpg_error_t failure_code;
+
+  /* Error encountered during the export.  */
+  gpg_error_t err;
+
 } *op_data_t;
 
 
@@ -88,6 +93,10 @@ export_status_handler (void *priv, gpgme_status_code_t code, char *args)
   op_data_t opd;
   const char *loc;
 
+  err = _gpgme_passphrase_status_handler (priv, code, args);
+  if (err)
+    return err;
+
   err = _gpgme_op_data_lookup (ctx, OPDATA_EXPORT, &hook, -1, NULL);
   opd = hook;
   if (err)
@@ -101,10 +110,15 @@ export_status_handler (void *priv, gpgme_status_code_t code, char *args)
         return err;
       else if (opd->err)
         ; /* We only want to report the first error.  */
-      else if (!strcmp (loc, "keyserver_send"))
+      else if (!strcmp (loc, "keyserver_send")
+               || !strcmp (loc, "export_keys.secret"))
         opd->err = err;
       break;
 
+    case GPGME_STATUS_FAILURE:
+      opd->failure_code = _gpgme_parse_failure (args);
+      break;
+
     default:
       break;
     }
@@ -113,22 +127,29 @@ export_status_handler (void *priv, gpgme_status_code_t code, char *args)
 
 
 static gpgme_error_t
-export_start (gpgme_ctx_t ctx, int synchronous, const char *pattern,
-	      gpgme_export_mode_t mode, gpgme_data_t keydata)
+check_mode (gpgme_export_mode_t mode, gpgme_protocol_t protocol,
+            gpgme_data_t keydata)
 {
-  gpgme_error_t err;
-  void *hook;
-  op_data_t opd;
-
   if ((mode & ~(GPGME_EXPORT_MODE_EXTERN
                 |GPGME_EXPORT_MODE_MINIMAL
                 |GPGME_EXPORT_MODE_SECRET
                 |GPGME_EXPORT_MODE_SSH
                 |GPGME_EXPORT_MODE_RAW
-                |GPGME_EXPORT_MODE_NOUID
-                |GPGME_EXPORT_MODE_PKCS12)))
+                |GPGME_EXPORT_MODE_PKCS12
+                |GPGME_EXPORT_MODE_SECRET_SUBKEY)))
     return gpg_error (GPG_ERR_INV_VALUE); /* Invalid flags in MODE.  */
 
+  if ((mode & GPGME_EXPORT_MODE_SSH))
+    {
+       if ((mode & (GPGME_EXPORT_MODE_EXTERN
+                    |GPGME_EXPORT_MODE_MINIMAL
+                    |GPGME_EXPORT_MODE_SECRET
+                    |GPGME_EXPORT_MODE_RAW
+                    |GPGME_EXPORT_MODE_PKCS12
+                    |GPGME_EXPORT_MODE_SECRET_SUBKEY)))
+          return gpg_error (GPG_ERR_INV_FLAG);  /* Combination not allowed. */
+    }
+
   if ((mode & GPGME_EXPORT_MODE_SECRET))
     {
       if ((mode & GPGME_EXPORT_MODE_EXTERN))
@@ -137,11 +158,17 @@ export_start (gpgme_ctx_t ctx, int synchronous, const char *pattern,
           && (mode & GPGME_EXPORT_MODE_PKCS12))
         return gpg_error (GPG_ERR_INV_FLAG);  /* Combination not allowed. */
 
-      if (ctx->protocol != GPGME_PROTOCOL_CMS
+      if (protocol != GPGME_PROTOCOL_CMS
           && (mode & (GPGME_EXPORT_MODE_RAW|GPGME_EXPORT_MODE_PKCS12)))
         return gpg_error (GPG_ERR_INV_FLAG);  /* Only supported for X.509.  */
     }
 
+  if ((mode & GPGME_EXPORT_MODE_SECRET_SUBKEY))
+    {
+      if ((mode & GPGME_EXPORT_MODE_EXTERN))
+        return gpg_error (GPG_ERR_INV_FLAG);  /* Combination not allowed. */
+    }
+
   if ((mode & GPGME_EXPORT_MODE_EXTERN))
     {
       if (keydata)
@@ -153,6 +180,22 @@ export_start (gpgme_ctx_t ctx, int synchronous, const char *pattern,
         return gpg_error (GPG_ERR_INV_VALUE);
     }
 
+  return 0;
+}
+
+
+static gpgme_error_t
+export_start (gpgme_ctx_t ctx, int synchronous, const char *pattern,
+	      gpgme_export_mode_t mode, gpgme_data_t keydata)
+{
+  gpgme_error_t err;
+  void *hook;
+  op_data_t opd;
+
+  err = check_mode (mode, ctx->protocol, keydata);
+  if (err)
+    return err;
+
   err = _gpgme_op_reset (ctx, synchronous);
   if (err)
     return err;
@@ -224,37 +267,9 @@ export_ext_start (gpgme_ctx_t ctx, int synchronous, const char *pattern[],
   void *hook;
   op_data_t opd;
 
-  if ((mode & ~(GPGME_EXPORT_MODE_EXTERN
-                |GPGME_EXPORT_MODE_MINIMAL
-                |GPGME_EXPORT_MODE_SECRET
-                |GPGME_EXPORT_MODE_SSH
-                |GPGME_EXPORT_MODE_RAW
-                |GPGME_EXPORT_MODE_PKCS12)))
-    return gpg_error (GPG_ERR_INV_VALUE); /* Invalid flags in MODE.  */
-
-  if ((mode & GPGME_EXPORT_MODE_SECRET))
-    {
-      if ((mode & GPGME_EXPORT_MODE_EXTERN))
-        return gpg_error (GPG_ERR_INV_FLAG);  /* Combination not allowed. */
-      if ((mode & GPGME_EXPORT_MODE_RAW)
-          && (mode & GPGME_EXPORT_MODE_PKCS12))
-        return gpg_error (GPG_ERR_INV_FLAG);  /* Combination not allowed. */
-
-      if (ctx->protocol != GPGME_PROTOCOL_CMS
-          && (mode & (GPGME_EXPORT_MODE_RAW|GPGME_EXPORT_MODE_PKCS12)))
-        return gpg_error (GPG_ERR_INV_FLAG);  /* Only supported for X.509.  */
-    }
-
-  if ((mode & GPGME_EXPORT_MODE_EXTERN))
-    {
-      if (keydata)
-        return gpg_error (GPG_ERR_INV_VALUE);
-    }
-  else
-    {
-      if (!keydata)
-        return gpg_error (GPG_ERR_INV_VALUE);
-    }
+  err = check_mode (mode, ctx->protocol, keydata);
+  if (err)
+    return err;
 
   err = _gpgme_op_reset (ctx, synchronous);
   if (err)
@@ -350,7 +365,7 @@ gpgme_op_export_ext (gpgme_ctx_t ctx, const char *pattern[],
           err = _gpgme_op_data_lookup (ctx, OPDATA_EXPORT, &hook, -1, NULL);
           opd = hook;
           if (!err)
-            err = opd->err;
+            err = opd->err ? opd->err : opd->failure_code;
         }
     }
 
@@ -372,6 +387,11 @@ export_keys_start (gpgme_ctx_t ctx, int synchronous, gpgme_key_t keys[],
   if (!keys)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if ((mode & GPGME_EXPORT_MODE_SECRET_SUBKEY))
+    {
+      return gpg_error (GPG_ERR_INV_FLAG);
+    }
+
   /* Create a list of pattern from the keys.  */
   for (idx=nkeys=0; keys[idx]; idx++)
     if (keys[idx]->protocol == ctx->protocol)
@@ -490,7 +510,7 @@ gpgme_op_export_keys (gpgme_ctx_t ctx,
           err = _gpgme_op_data_lookup (ctx, OPDATA_EXPORT, &hook, -1, NULL);
           opd = hook;
           if (!err)
-            err = opd->err;
+            err = opd->err ? opd->err : opd->failure_code;
         }
     }
 
