@@ -1,23 +1,23 @@
 /* sign.c - Signing function.
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004, 2007 g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2002, 2003, 2004, 2007 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -39,6 +39,15 @@ typedef struct
 {
   struct _gpgme_op_sign_result result;
 
+  /* The error code from a FAILURE status line or 0.  */
+  gpg_error_t failure_code;
+
+  /* The fingerprint from the last KEY_CONSIDERED status line.  */
+  char *kc_fpr;
+
+  /* The flags from the last KEY_CONSIDERED status line.  */
+  unsigned int kc_flags;
+
   /* A pointer to the next pointer of the last invalid signer in
      the list.  This makes appending new invalid signers painless
      while preserving the order.  */
@@ -83,6 +92,7 @@ release_op_data (void *hook)
     }
 
   release_signatures (opd->result.signatures);
+  free (opd->kc_fpr);
 }
 
 
@@ -97,13 +107,13 @@ gpgme_op_sign_result (gpgme_ctx_t ctx)
   unsigned int inv_signers = 0;
   unsigned int signatures = 0;
 
-  TRACE_BEG (DEBUG_CTX, "gpgme_op_sign_result", ctx);
+  TRACE_BEG (DEBUG_CTX, "gpgme_op_sign_result", ctx, "");
 
   err = _gpgme_op_data_lookup (ctx, OPDATA_SIGN, &hook, -1, NULL);
   opd = hook;
   if (err || !opd)
     {
-      TRACE_SUC0 ("result=(null)");
+      TRACE_SUC ("result=(null)");
       return NULL;
     }
 
@@ -115,12 +125,12 @@ gpgme_op_sign_result (gpgme_ctx_t ctx)
   if (gpgme_signers_count (ctx)
       && signatures + inv_signers != gpgme_signers_count (ctx))
     {
-      /* In this case at least one signatures was not created perhaps
+      /* In this case at least one signature was not created perhaps
          due to a bad passphrase etc.  Thus the entire message is
          broken and should not be used.  We add the already created
          signatures to the invalid signers list and thus this case can
          be detected.  */
-      TRACE_LOG3 ("result: invalid signers: %u, signatures: %u, count: %u",
+      TRACE_LOG  ("result: invalid signers: %u, signatures: %u, count: %u",
                   inv_signers, signatures, gpgme_signers_count (ctx));
 
       for (sig = opd->result.signatures; sig; sig = sig->next)
@@ -128,7 +138,7 @@ gpgme_op_sign_result (gpgme_ctx_t ctx)
           key = calloc (1, sizeof *key);
           if (!key)
             {
-              TRACE_SUC0 ("out of core; result=(null)");
+              TRACE_SUC ("out of core; result=(null)");
               return NULL;
             }
           if (sig->fpr)
@@ -137,7 +147,7 @@ gpgme_op_sign_result (gpgme_ctx_t ctx)
               if (!key->fpr)
                 {
                   free (key);
-                  TRACE_SUC0 ("out of core; result=(null)");
+                  TRACE_SUC ("out of core; result=(null)");
                   return NULL;
                 }
             }
@@ -160,24 +170,24 @@ gpgme_op_sign_result (gpgme_ctx_t ctx)
 
   if (_gpgme_debug_trace())
     {
-      TRACE_LOG2 ("result: invalid signers: %i, signatures: %i",
+      TRACE_LOG  ("result: invalid signers: %i, signatures: %i",
 		  inv_signers, signatures);
       for (inv_key=opd->result.invalid_signers; inv_key; inv_key=inv_key->next)
 	{
-	  TRACE_LOG3 ("result: invalid signer: fpr=%s, reason=%s <%s>",
+	  TRACE_LOG  ("result: invalid signer: fpr=%s, reason=%s <%s>",
 		      inv_key->fpr, gpgme_strerror (inv_key->reason),
 		      gpgme_strsource (inv_key->reason));
 	}
       for (sig = opd->result.signatures; sig; sig = sig->next)
 	{
-	  TRACE_LOG6 ("result: signature: type=%i, pubkey_algo=%i, "
+	  TRACE_LOG  ("result: signature: type=%i, pubkey_algo=%i, "
 		      "hash_algo=%i, timestamp=%li, fpr=%s, sig_class=%i",
 		      sig->type, sig->pubkey_algo, sig->hash_algo,
 		      sig->timestamp, sig->fpr, sig->sig_class);
 	}
    }
 
-  TRACE_SUC1 ("result=%p", &opd->result);
+  TRACE_SUC ("result=%p", &opd->result);
   return &opd->result;
 }
 
@@ -241,7 +251,16 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp,
     }
   args = tail;
 
+  /* strtol has been used wrongly here.  We can't change this anymore
+   * but we now take care of the 0x1f class which would otherwise let
+   * us run into an error.  */
   sig->sig_class = strtol (args, &tail, 0);
+  if (!errno && args != tail && sig->sig_class == 1
+      && (*tail == 'F' || *tail == 'f'))
+    {
+      tail++;
+      sig->sig_class = 131; /* Arbitrary unused value in rfc4880. */
+    }
   sig->class = sig->sig_class;
   sig->_obsolete_class = sig->sig_class;
   if (errno || args == tail || *tail != ' ')
@@ -313,18 +332,38 @@ _gpgme_sign_status_handler (void *priv, gpgme_status_code_t code, char *args)
       opd->last_sig_p = &(*opd->last_sig_p)->next;
       break;
 
+    case GPGME_STATUS_KEY_CONSIDERED:
+      /* This is emitted during gpg's key lookup to give information
+       * about the lookup results.  We store the last one so it can be
+       * used in connection with INV_RECP.  */
+      free (opd->kc_fpr);
+      opd->kc_fpr = NULL;
+      err = _gpgme_parse_key_considered (args, &opd->kc_fpr, &opd->kc_flags);
+      if (err)
+        return err;
+      break;
+
     case GPGME_STATUS_INV_RECP:
       if (opd->inv_sgnr_seen && opd->ignore_inv_recp)
         break;
-      /* FALLTROUGH */
+      /* FALLTHROUGH */
     case GPGME_STATUS_INV_SGNR:
       if (code == GPGME_STATUS_INV_SGNR)
         opd->inv_sgnr_seen = 1;
-      err = _gpgme_parse_inv_recp (args, opd->last_signer_p);
+      free (opd->kc_fpr);
+      opd->kc_fpr = NULL;
+      err = _gpgme_parse_inv_recp (args, 1, opd->kc_fpr, opd->kc_flags,
+                                   opd->last_signer_p);
       if (err)
 	return err;
 
       opd->last_signer_p = &(*opd->last_signer_p)->next;
+      free (opd->kc_fpr);
+      opd->kc_fpr = NULL;
+      break;
+
+    case GPGME_STATUS_FAILURE:
+      opd->failure_code = _gpgme_parse_failure (args);
       break;
 
     case GPGME_STATUS_EOF:
@@ -335,7 +374,12 @@ _gpgme_sign_status_handler (void *priv, gpgme_status_code_t code, char *args)
 	err = gpg_error (GPG_ERR_UNUSABLE_SECKEY);
       else if (!opd->sig_created_seen
                && ctx->protocol != GPGME_PROTOCOL_UISERVER)
-	err = gpg_error (GPG_ERR_GENERAL);
+	err = opd->failure_code? opd->failure_code:gpg_error (GPG_ERR_GENERAL);
+      break;
+
+    case GPGME_STATUS_INQUIRE_MAXLEN:
+      if (ctx->status_cb && !ctx->full_status)
+        err = ctx->status_cb (ctx->status_cb_value, "INQUIRE_MAXLEN", args);
       break;
 
     default:
@@ -369,6 +413,7 @@ sign_init_result (gpgme_ctx_t ctx, int ignore_inv_recp)
   opd = hook;
   if (err)
     return err;
+  opd->failure_code = 0;
   opd->last_signer_p = &opd->result.invalid_signers;
   opd->last_sig_p = &opd->result.signatures;
   opd->ignore_inv_recp = !!ignore_inv_recp;
@@ -386,7 +431,7 @@ _gpgme_op_sign_init_result (gpgme_ctx_t ctx)
 
 static gpgme_error_t
 sign_start (gpgme_ctx_t ctx, int synchronous, gpgme_data_t plain,
-	    gpgme_data_t sig, gpgme_sig_mode_t mode)
+	    gpgme_data_t sig, gpgme_sig_mode_t flags)
 {
   gpgme_error_t err;
 
@@ -401,8 +446,9 @@ sign_start (gpgme_ctx_t ctx, int synchronous, gpgme_data_t plain,
   if (err)
     return err;
 
-  if (mode != GPGME_SIG_MODE_NORMAL && mode != GPGME_SIG_MODE_DETACH
-      && mode != GPGME_SIG_MODE_CLEAR)
+  if (flags & ~(GPGME_SIG_MODE_DETACH
+                |GPGME_SIG_MODE_CLEAR
+                |GPGME_SIG_MODE_ARCHIVE))
     return gpg_error (GPG_ERR_INV_VALUE);
 
   if (!plain)
@@ -413,7 +459,7 @@ sign_start (gpgme_ctx_t ctx, int synchronous, gpgme_data_t plain,
   if (ctx->passphrase_cb)
     {
       err = _gpgme_engine_set_command_handler
-	(ctx->engine, _gpgme_passphrase_command_handler, ctx, NULL);
+	(ctx->engine, _gpgme_passphrase_command_handler, ctx);
       if (err)
 	return err;
     }
@@ -421,7 +467,7 @@ sign_start (gpgme_ctx_t ctx, int synchronous, gpgme_data_t plain,
   _gpgme_engine_set_status_handler (ctx->engine, sign_status_handler,
 				    ctx);
 
-  return _gpgme_engine_op_sign (ctx->engine, plain, sig, mode, ctx->use_armor,
+  return _gpgme_engine_op_sign (ctx->engine, plain, sig, flags, ctx->use_armor,
 				ctx->use_textmode, ctx->include_certs,
 				ctx /* FIXME */);
 }
@@ -430,16 +476,16 @@ sign_start (gpgme_ctx_t ctx, int synchronous, gpgme_data_t plain,
 /* Sign the plaintext PLAIN and store the signature in SIG.  */
 gpgme_error_t
 gpgme_op_sign_start (gpgme_ctx_t ctx, gpgme_data_t plain, gpgme_data_t sig,
-		     gpgme_sig_mode_t mode)
+		     gpgme_sig_mode_t flags)
 {
   gpg_error_t err;
-  TRACE_BEG3 (DEBUG_CTX, "gpgme_op_sign_start", ctx,
-	      "plain=%p, sig=%p, mode=%i", plain, sig, mode);
+  TRACE_BEG  (DEBUG_CTX, "gpgme_op_sign_start", ctx,
+	      "plain=%p, sig=%p, flags=%i", plain, sig, flags);
 
   if (!ctx)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
 
-  err = sign_start (ctx, 0, plain, sig, mode);
+  err = sign_start (ctx, 0, plain, sig, flags);
   return TRACE_ERR (err);
 }
 
@@ -447,17 +493,17 @@ gpgme_op_sign_start (gpgme_ctx_t ctx, gpgme_data_t plain, gpgme_data_t sig,
 /* Sign the plaintext PLAIN and store the signature in SIG.  */
 gpgme_error_t
 gpgme_op_sign (gpgme_ctx_t ctx, gpgme_data_t plain, gpgme_data_t sig,
-	       gpgme_sig_mode_t mode)
+	       gpgme_sig_mode_t flags)
 {
   gpgme_error_t err;
 
-  TRACE_BEG3 (DEBUG_CTX, "gpgme_op_sign", ctx,
-	      "plain=%p, sig=%p, mode=%i", plain, sig, mode);
+  TRACE_BEG  (DEBUG_CTX, "gpgme_op_sign", ctx,
+	      "plain=%p, sig=%p, flags=%i", plain, sig, flags);
 
   if (!ctx)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
 
-  err = sign_start (ctx, 1, plain, sig, mode);
+  err = sign_start (ctx, 1, plain, sig, flags);
   if (!err)
     err = _gpgme_wait_one (ctx);
   return TRACE_ERR (err);
