@@ -1,21 +1,22 @@
 /* run-keylist.c  - Helper to show a key listing.
-   Copyright (C) 2008, 2009 g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <https://www.gnu.org/licenses/>.
-*/
+ * Copyright (C) 2008, 2009 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 /* We need to include config.h so that we know whether we are building
    with large file system (LFS) support. */
@@ -54,12 +55,15 @@ show_usage (int ex)
          "  --tofu           use GPGME_KEYLIST_MODE_TOFU\n"
          "  --sig-notations  use GPGME_KEYLIST_MODE_SIG_NOTATIONS\n"
          "  --ephemeral      use GPGME_KEYLIST_MODE_EPHEMERAL\n"
+         "  --v5fpr          use GPGME_KEYLIST_MODE_V5FPR\n"
          "  --validate       use GPGME_KEYLIST_MODE_VALIDATE\n"
          "  --import         import all keys\n"
          "  --offline        use offline mode\n"
+         "  --no-trust-check disable automatic trust database check\n"
          "  --from-file      list all keys in the given file\n"
          "  --from-wkd       list key from a web key directory\n"
          "  --require-gnupg  required at least the given GnuPG version\n"
+         "  --trust-model    use the specified trust-model\n"
          , stderr);
   exit (ex);
 }
@@ -101,9 +105,11 @@ main (int argc, char **argv)
   gpgme_protocol_t protocol = GPGME_PROTOCOL_OpenPGP;
   int only_secret = 0;
   int offline = 0;
+  int no_trust_check = 0;
   int from_file = 0;
   int from_wkd = 0;
   gpgme_data_t data = NULL;
+  char *trust_model = NULL;
 
 
   if (argc)
@@ -179,6 +185,11 @@ main (int argc, char **argv)
           mode |= GPGME_KEYLIST_MODE_WITH_SECRET;
           argc--; argv++;
         }
+      else if (!strcmp (*argv, "--v5fpr"))
+        {
+          mode |= GPGME_KEYLIST_MODE_WITH_V5FPR;
+          argc--; argv++;
+        }
       else if (!strcmp (*argv, "--import"))
         {
           import = 1;
@@ -189,6 +200,11 @@ main (int argc, char **argv)
           offline = 1;
           argc--; argv++;
         }
+      else if (!strcmp (*argv, "--no-trust-check"))
+        {
+          no_trust_check = 1;
+          argc--; argv++;
+        }
       else if (!strcmp (*argv, "--from-file"))
         {
           from_file = 1;
@@ -208,6 +224,14 @@ main (int argc, char **argv)
           mode |= GPGME_KEYLIST_MODE_LOCATE;
           from_wkd = 1;
         }
+      else if (!strcmp (*argv, "--trust-model"))
+        {
+          argc--; argv++;
+          if (!argc)
+            show_usage (1);
+          trust_model = strdup (*argv);
+          argc--; argv++;
+        }
       else if (!strncmp (*argv, "--", 2))
         show_usage (1);
     }
@@ -227,6 +251,18 @@ main (int argc, char **argv)
 
   gpgme_set_offline (ctx, offline);
 
+  if (no_trust_check)
+    {
+      err = gpgme_set_ctx_flag (ctx, "no-auto-check-trustdb", "1");
+      fail_if_err (err);
+    }
+
+  if (trust_model)
+    {
+      err = gpgme_set_ctx_flag (ctx, "trust-model", trust_model);
+      fail_if_err (err);
+    }
+
   if (from_wkd)
     {
       err = gpgme_set_ctx_flag (ctx, "auto-key-locate",
@@ -255,16 +291,16 @@ main (int argc, char **argv)
       int nsigs;
 
       printf ("keyid   : %s\n", key->subkeys?nonnull (key->subkeys->keyid):"?");
-      printf ("fpr     : %s\n", key->subkeys?nonnull (key->subkeys->fpr):"?");
-      if (key->subkeys && key->subkeys->keygrip)
-        printf ("grip    : %s\n", key->subkeys->keygrip);
-      if (key->subkeys && key->subkeys->curve)
-            printf ("curve   : %s\n", key->subkeys->curve);
-      printf ("caps    : %s%s%s%s\n",
+      printf ("can_cap : %s%s%s%s\n",
               key->can_encrypt? "e":"",
               key->can_sign? "s":"",
               key->can_certify? "c":"",
               key->can_authenticate? "a":"");
+      printf ("has_cap : %s%s%s%s\n",
+              key->has_encrypt? "e":"",
+              key->has_sign? "s":"",
+              key->has_certify? "c":"",
+              key->has_authenticate? "a":"");
       printf ("flags   :%s%s%s%s%s%s%s%s\n",
               key->secret? " secret":"",
               key->revoked? " revoked":"",
@@ -277,28 +313,31 @@ main (int argc, char **argv)
       printf ("upd     : %lu (%u)\n", key->last_update, key->origin);
 
       subkey = key->subkeys;
-      if (subkey)
-        subkey = subkey->next;
-      for (nsub=1; subkey; subkey = subkey->next, nsub++)
+      for (nsub=0; subkey; subkey = subkey->next, nsub++)
         {
           printf ("fpr   %2d: %s\n", nsub, nonnull (subkey->fpr));
+          if (subkey->v5fpr)
+            printf ("v5fpr %2d: %s\n", nsub, nonnull (subkey->v5fpr));
           if (subkey->keygrip)
             printf ("grip  %2d: %s\n", nsub, subkey->keygrip);
           if (subkey->curve)
             printf ("curve %2d: %s\n", nsub, subkey->curve);
-          printf ("caps  %2d: %s%s%s%s\n",
+          printf ("caps  %2d: %s%s%s%s%s%s\n",
                   nsub,
                   subkey->can_encrypt? "e":"",
                   subkey->can_sign? "s":"",
                   subkey->can_certify? "c":"",
-                  subkey->can_authenticate? "a":"");
-          printf ("flags %2d:%s%s%s%s%s%s%s%s\n",
+                  subkey->can_authenticate? "a":"",
+                  subkey->can_renc? "r":"",
+                  subkey->can_timestamp? "t":"");
+          printf ("flags %2d:%s%s%s%s%s%s%s%s%s\n",
                   nsub,
                   subkey->secret? " secret":"",
                   subkey->revoked? " revoked":"",
                   subkey->expired? " expired":"",
                   subkey->disabled? " disabled":"",
                   subkey->invalid? " invalid":"",
+                  subkey->is_group_owned? " group":"",
                   subkey->is_qualified? " qualified":"",
                   subkey->is_de_vs? " de-vs":"",
                   subkey->is_cardkey? " cardkey":"");
@@ -313,6 +352,8 @@ main (int argc, char **argv)
             printf ("    name: %s\n", uid->name);
           if (uid->comment)
             printf ("   cmmnt: %s\n", uid->comment);
+          if (uid->uidhash)
+            printf (" uidhash: %s\n", uid->uidhash);
           printf ("     upd: %lu (%u)\n", uid->last_update, uid->origin);
           printf ("   valid: %s\n",
                   uid->validity == GPGME_VALIDITY_UNKNOWN? "unknown":
@@ -350,6 +391,9 @@ main (int argc, char **argv)
               printf ("     created: %s\n", isotimestr(ks->timestamp));
               printf ("     expires: %s\n", isotimestr(ks->expires));
               printf ("       class: %x\n", ks->sig_class);
+              printf (" trust depth: %u\n", ks->trust_depth);
+              printf (" trust value: %u\n", ks->trust_value);
+              printf (" trust scope: %s\n", nonnull (ks->trust_scope));
             }
         }
 
@@ -401,6 +445,8 @@ main (int argc, char **argv)
   for (keyidx=0; keyarray[keyidx]; keyidx++)
     gpgme_key_unref (keyarray[keyidx]);
 
+  free (trust_model);
+
   gpgme_release (ctx);
   return 0;
 }
